-----------------------------------------------------------------------------------------------
-- Client Lua Script for SpiralStaircaseHelper
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelper Module Definition
-----------------------------------------------------------------------------------------------
local SpiralStaircaseHelper = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SpiralStaircaseHelper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.nDirection = 1
	self.nPillarX = 0
	self.nPillarZ = 0
	self.nStairX = 0
	self.nStairZ = 0
	self.nHeight = 0
	self.nDHeight = 0
	self.nRoll = 0
	self.nTheta = 0
	self.nDRoll = 0
	self.nAmount = 0
	self.nRadius = 0

    return o
end

function SpiralStaircaseHelper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelper OnLoad
-----------------------------------------------------------------------------------------------
function SpiralStaircaseHelper:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SpiralStaircaseHelper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelper OnDocLoaded
-----------------------------------------------------------------------------------------------
function SpiralStaircaseHelper:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SpiralStaircaseHelperForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
		self.wndInfo = self.wndMain:FindChild("Info")
		self.wndInput = self.wndMain:FindChild("InputArea")
	    self.wndMain:Show(false, true)
		self.wndInfo:Show(false)
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("spiralstairs", "OnSpiralStaircaseHelperOn", self)


		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelper Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/spiralstairs"
function SpiralStaircaseHelper:OnSpiralStaircaseHelperOn()
	self.wndMain:Invoke() -- show the window
	self.wndInput:FindChild("Direction"):SetCheck(self.nDirection == 1)
end


function SpiralStaircaseHelper:Calculate()
	for n = 0, self.nAmount - 1, 1 do
		self:ListCalculate()
	end
end


function SpiralStaircaseHelper:ListCalculate()
	i = {}
	
	local x = self.nPillarX + (self.nRadius * math.cos(math.rad(self.nTheta)))
	local y = self.nHeight
	local z = self.nPillarZ + (self.nRadius * math.sin(math.rad(self.nTheta)))
	local roll = self.nRoll
	local altroll = self:DirectionHelper(roll - 180)
	
	i.x = string.format("X: %.2f", x)
	i.y = string.format("Y: %.2f", y)
	i.z = string.format("Z: %.2f", z)
	i.roll = string.format("Roll: %.2f or %.2f", roll, altroll)
	
	self.nHeight = self.nHeight + self.nDHeight
	self.nRoll = self:DirectionHelper(self.nRoll + -1 * self.nDirection * self.nDRoll)
	self.nTheta = self:DirectionHelper(self.nTheta + self.nDirection * self.nDRoll)
	
	self:AddItem(i)
end


function SpiralStaircaseHelper:DirectionHelper(nRoll)
	nRoll = nRoll % 360
	if nRoll > 180 then
		nRoll = nRoll - 360
	end
	return nRoll
end


-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelperForm Functions
-----------------------------------------------------------------------------------------------
-- when the Cancel button is clicked
function SpiralStaircaseHelper:OnCancel()
	self.wndMain:Close() -- hide the window
end


function SpiralStaircaseHelper:Clockwise( wndHandler, wndControl, eMouseButton )
	self.nDirection = 1
end

function SpiralStaircaseHelper:Counterclockwise( wndHandler, wndControl, eMouseButton )
	self.nDirection = -1
end

function SpiralStaircaseHelper:OnCalculate(wndHandler, wndControl, eMouseButton)
	self.nPillarX = tonumber(self.wndInput:FindChild("PillarX"):GetText())
	self.nPillarZ = tonumber(self.wndInput:FindChild("PillarZ"):GetText())
	self.nStairX = tonumber(self.wndInput:FindChild("StairX"):GetText())
	self.nStairZ = tonumber(self.wndInput:FindChild("StairZ"):GetText())
	self.nHeight = tonumber(self.wndInput:FindChild("Height"):GetText())
	self.nDHeight = tonumber(self.wndInput:FindChild("DHeight"):GetText())
	self.nRoll = self:DirectionHelper(tonumber(self.wndInput:FindChild("Roll"):GetText()))
	self.nDRoll = tonumber(self.wndInput:FindChild("DRoll"):GetText())
	self.nAmount = tonumber(self.wndInput:FindChild("Amount"):GetText())
	
	self.nRadius = math.sqrt(math.pow(self.nStairX - self.nPillarX, 2) + math.pow(self.nStairZ - self.nPillarZ, 2))
	self.nTheta = math.deg(math.acos((self.nStairX - self.nPillarX) / self.nRadius))
	if self.nStairZ - self.nPillarZ < 0 then
		self.nTheta = self.nTheta * -1
	end
	
	self:DestroyItemList()
	self:Calculate()
end

function SpiralStaircaseHelper:OnInfo( wndHandler, wndControl, eMouseButton )
	self.wndInfo:Show(true)
end

function SpiralStaircaseHelper:OnCloseInfo( wndHandler, wndControl, eMouseButton )
	self.wndInfo:Show(false)
end

-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
-- clear the item list
function SpiralStaircaseHelper:DestroyItemList()
	-- destroy all the wnd inside the list
	for idx,wnd in ipairs(self.wndItemList:GetChildren()) do
		if wnd ~= nil then wnd:Destroy() end
	end
end

-- add an item into the item list
function SpiralStaircaseHelper:AddItem(i)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	
	wnd:FindChild("X"):SetText(i.x)
	wnd:FindChild("Y"):SetText(i.y)
	wnd:FindChild("Z"):SetText(i.z)
	wnd:FindChild("Roll"):SetText(i.roll)
	
	self.wndItemList:ArrangeChildrenVert(0)
end


---------------------------------------------------------------------------------------------------
-- ListItem Functions
---------------------------------------------------------------------------------------------------

function SpiralStaircaseHelper:OnPlacedCheck( wndHandler, wndControl, eMouseButton )
	for i,wnd in pairs(wndHandler:GetParent():GetChildren()) do
		wnd:SetTextColor("darkgray")
	end
end

function SpiralStaircaseHelper:OnPlacedUncheck( wndHandler, wndControl, eMouseButton )
	for i,wnd in pairs(wndHandler:GetParent():GetChildren()) do
		wnd:SetTextColor("UI_WindowTextDefault")
	end
end

-----------------------------------------------------------------------------------------------
-- SpiralStaircaseHelper Instance
-----------------------------------------------------------------------------------------------
local SpiralStaircaseHelperInst = SpiralStaircaseHelper:new()
SpiralStaircaseHelperInst:Init()
